﻿using System;

namespace Ruru.Game
{
    /// <summary>
    /// 타자
    /// </summary>
    public class Hitter
        : Ruru.Game.IBaseBallPlayer
    {
        /// <summary>
        /// 공들
        /// </summary>
        private int[] ball;
        /// <summary>
        /// 랜덤시드값 (기본 37)
        /// </summary>
        private static int seed = 37;
        /// <summary>
        /// 랜덤
        /// </summary>
        private Random r;
        /// <summary>
        /// 횟수
        /// </summary>
        private int num;
        /// <summary>
        /// 게임 숫자범위
        /// </summary>
        private int range;

        /// <summary>
        /// 기본생성자
        /// </summary>
        public Hitter() : this(10, 3) { }

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="range">범위</param>
        /// <param name="num">숫자갯수</param>
        public Hitter(int range, int num)
        {
            this.range = range;
            this.num = num;
            this.Init(range, num);
        }

        /// <summary>
        /// 소멸자
        /// </summary>
        ~Hitter()
        {
        }

        /// <summary>
        /// int[] ball 에 대한 프로퍼티
        /// </summary>
        public int[] Ball
        {
            get { return this.ball; }
        }

        /// <summary>
        /// 값을 초기화한다
        /// </summary>
        /// <param name="range">범위</param>
        /// <param name="num">갯수</param>
        public void Init(int range, int num)
        {
            ball = new int[num];
            r = new Random(DateTime.Now.Millisecond + seed++);

            for (int i = 0; i < ball.Length; i++)
                ball[i] = -1;
        }

        /// <summary>
        /// 숫자하나를 입력받는다
        /// </summary>
        /// <returns></returns>
        public int MakeOne()
        {
            string snum = string.Empty;
            int temp = 0;

            Console.WriteLine("{0} ~ {1}의 정수를 입력하세요", 0, range - 1);
            snum = Console.ReadLine();

            try
            {
                temp = int.Parse(snum.Trim());
            }
            catch
            {
                throw new Exception("{0} ~ {1}의 정수를 입력하세요", 0, range - 1);
            }
            if (temp < 0 || temp > range - 1)
            {
                throw new Exception("범위초과");
            }

            return temp;
        }

        /// <summary>
        /// 지정된 갯수만큼 읽어들인다.
        /// </summary>
        public void Make()
        {
            int count = 0;
            while (count != 3)
            {
                int temp = -1;

                try
                {
                    temp = this.MakeOne();
                }
                catch(Exception ee)
                {
                    Console.WriteLine("다시입력하세요 : " + ee.Message);
                }
                if (!this.HasSame(temp))
                {
                    ball[count++] = temp;
                }
                else
                {
                    Console.WriteLine("이미 입력된 값입니다.");
                }

            }
        }

        /// <summary>
        /// 새로 게임 생성
        /// </summary>
        public void ReMake()
        {
            this.Init(range, num);
            this.Make();
        }

        /// <summary>
        /// 지정된 숫자가 맴버 배열에 값이 있는가 없는가를 체크
        /// </summary>
        /// <param name="n">입력 수</param>
        /// <returns>값이 있는가 없는가</returns>
        public bool HasSame(int n)
        {
            bool hasS = false;

            for (int i = 0; i < ball.Length; i++)
            {
                if (ball[i] == n)
                {
                    hasS = true;
                    break;
                }
            }

            return hasS;
        }

    }
}
